import os, sys
import subprocess as sp

# image input
input_path = os.path.normcase(sys.argv[1])
str_E_measured = sys.argv[2]
output_path = os.path.normcase(sys.argv[3])


# calculate E_image
command = ["pcomb", "-e", "lo=L*Sang*cosCos;L=179*li(1);Sang=S(1);cosCos=Dy(1);", "-o", input_path, "|", "pvalue", "-d", "-b", "-h", "-H", "|", "total"]
proc = sp.Popen(command, stdout=sp.PIPE, shell=True)
str_E_image, err = proc.communicate()
str_E_image = str_E_image.rstrip()
E_image = float(str_E_image)
print('Calculated image illuminance is %.1f lx.' % E_image)


# find brightest and dimmest pixel in image
command = ["pextrem", "-o", input_path, "|", "rcalc", "-e", "$1=($3*0.2127+$4*0.7152+$5*0.0722)*179.0", ">", "extremes.txt"]
proc = sp.Popen(command, stdout=sp.PIPE, shell=True)
proc.wait()
f = open("extremes.txt", 'r')
line_extrem = f.readline()
line_extrem = f.readline()
line_extrem = line_extrem.rstrip()
line_extrem = line_extrem.replace("\t", " ")
f.close()
os.remove("extremes.txt")


# calculate 95% max luminance
extreme = (float(line_extrem))
str_extreme = str(0.95 * extreme)
print('Highest luminance recorded in the image is %.1f cd/m2; therefore, the overflow threshold is %.1f cd/m2.' % (extreme, extreme * 0.95))


# find E_contrib of pixels in the overflow threshold
command = ['pcomb', '-e', 'lo=if(li(1)-' + str_extreme + '/179,illContrib,0);illContrib=L*Sang*cosCor;L=179*li(1);Sang=S(1);cosCor=Dy(1);', '-o', input_path, '|', 'pvalue', '-d', '-b', '-h', '-H', '|', 'total', '>', 'E_contrib.txt']
proc = sp.Popen(command, stdout=sp.PIPE, shell=True)
proc.wait()  
f = open('E_contrib.txt', 'r')
str_E_contrib = f.readline().rstrip()
E_contrib = float(str_E_contrib)
f.close()
os.remove('E_contrib.txt')
print('Contribution of pixels within the overflow threshold (E_contrib) is %.1f lx.' % E_contrib)

# find E_potential of pixels in the threshold
command = ['pcomb', '-e', 'lo=if(li(1)-' + str_extreme + '/179,illPoten,0);illPoten=Sang*cosCor;Sang=S(1);cosCor=Dy(1);', '-o', input_path, '|', 'pvalue', '-d', '-b', '-h', '-H', '|', 'total', '>', 'E_potential.txt']
proc = sp.Popen(command, stdout=sp.PIPE, shell=True)
proc.wait()
f = open('E_potential.txt', 'r')
str_E_potential = f.readline().rstrip()
E_potential = float(str_E_potential)
f.close()
os.remove('E_potential.txt')
print('Potential contribution of pixels above the overflow threshold (sum of solid angle plus cosine correction) is %.5f str.' % E_potential)


# calculate new luminance of pixels
L_new = (float(str_E_measured) - (E_image - E_contrib)) / E_potential
str_L_new = str(L_new)
print('These pixels will be adjusted to a luminance value of %.1f cd/m2.' % L_new) 

coef= L_new / (extreme * 0.95) 
print('RGB values in overflow pixels will be adjusted with a coefficient value of %.5f.' % coef) 
str_coef = str(coef)

# generate new image with adjusted luminances
command = ['pcomb', '-e', 'ro=if(li(1)-' + str_extreme + '/179,' + str_coef + '*ri(1), ri(1))', '-e', 'go=if(li(1)-' + str_extreme + '/179,' + str_coef + '*gi(1), gi(1))', '-e', 'bo=if(li(1)-' + str_extreme + '/179,' + str_coef + '*bi(1), bi(1))', '-o', input_path, '>', 'temp.hdr']
proc = sp.Popen(command, stdout=sp.PIPE, shell=True)
proc.wait()
proc = sp.Popen(command, stdout=sp.PIPE, shell=True)
proc.wait()


# append proper view information
tmpfile = open("temp.hdr", "rb")
outfile = open(output_path, "wb")
for (n,line) in enumerate(tmpfile):
    outfile.write(line)
#    if n == 2:
#        outfile.write("VIEW= -vta -vh 180 -vv 180\n")
tmpfile.close()
outfile.close()
os.remove('temp.hdr')

print('%s successfully created from %s corrected to an illuminance value of %s lx.' % (output_path, input_path, str_E_measured))




