from PIL import Image
import os, sys
import numpy
import math

########################################################################################
# input is presumed to be a folder containing folders of images taken with a Sigma EX DG 8mm 1:3.5 lens.
# input will be cropped based on a Sigma EX DG 8mm 1:3.5 lens.
# output will be the same image [0-255] in an equi-angle projection with square ratio and metadata.
########################################################################################

base_folder = 'C:\\Users\\Bo\\Desktop\\example_set\\raw_ldr_photos\\'

def fbilinear(source, row, col):
	# source = numpy image base array, RGB image representation
	
	y = row
	x = col
	
	x0 = numpy.floor(x).astype(int)
	x1 = x0 + 1
	if x1 >= 800:
		x1 = 799
	y0 = numpy.floor(y).astype(int)
	y1 = y0 + 1
	if y1 >= 800:
		y1= 799
	
	aR = float(source[y0,x0,0])
	bR = float(source[y1,x1,0])
	cR = float(source[y0,x1,0])
	dR = float(source[y1,x1,0])
	aG = float(source[y0,x0,1])
	bG = float(source[y1,x1,1])
	cG = float(source[y0,x1,1])
	dG = float(source[y1,x1,1])
	aB = float(source[y0,x0,2])
	bB = float(source[y1,x1,2])
	cB = float(source[y0,x1,2])
	dB = float(source[y1,x1,2])
	
	sa = (x1-x) * (y1-y)
	sb = (x1-x) * (y-y0)
	sc = (x-x0) * (y1-y)
	sd = (x-x0) * (y-y0)
	
	r = sa*aR + sb*bR + sc*cR + sd*dR
	g = sa*aG + sb*bG + sc*cG + sd*dG
	b = sa*aB + sb*bB + sc*cB + sd*dB
	
	return numpy.asarray((r,g,b))


folder_list = os.listdir(base_folder)
for folder in folder_list:
	main_folder = base_folder + folder + '\\'
	output_folder = base_folder + folder + ' [vta]\\'
	try:
		os.makedirs(output_folder)
	except:
		pass
	
	working_folder = main_folder
	
	filelist = os.listdir(working_folder)
	
	for file in filelist:
		path = working_folder + file
		output_filename = output_folder + file

		### load image into memory
		# path = 'e:\\Luminous Overflow\\Pilot\\ND1 - 01\\18 No Filter\\5C3A0313.JPG'
		im = Image.open(path)
		### crop
		x_dim,y_dim = im.size # dimensions
	
		# locate centers
		x_center = 1573  # camera specific
		y_center = 1064  # camera specific 
		# bounds
		radius = 1050  # camera specific

		left_bound = x_center - radius
		right_bound = x_center + radius
		top_bound = y_center - radius
		bottom_bound = y_center + radius
		box = (left_bound, top_bound, right_bound, bottom_bound) # crop box
		cropped = im.crop(box) # crop

		### circular mask
		x_dim,y_dim = cropped.size # dimensions
		# radius = x_dim/2
		#for x in range(x_dim):
		#	for y in range(y_dim):
		#		dist = ( (x-radius+0.5)**2 + (y-radius+0.5)**2 )**0.5
		#		if dist > radius:
		#			cropped.putpixel((x,y), (0,0,0))
					
		### resize to 800 x 800
		size = (800,800)
		scaled = cropped.resize(size, resample=Image.BILINEAR) 

		### bilinear interpolate to angular projection
		tmp = numpy.array(scaled) # convert to numpy array for ease
		tform_ar = numpy.zeros((800,800,3))
		# equisolid projection
		# r=2*f*sin(theta/2)
		# equiangle projection
		# r = f*theta
		# r = distance from optical axis (mm)
		# f = focal length (mm)
		# theta = entrance angle

		for new_row in range(800):
			for new_col in range(800):
			
				px_dist_center = ( ((new_row+0.5)-400.0)**2 + ((new_col+0.5)-400.0)**2 )**0.5
				
				if px_dist_center < 400.0: # only if its within radius
					new_r = (px_dist_center / 400.0) * 12.56637061 # new_r in mm; 12.56637061 is ideal r_original based on equiangle type
					theta_rad = math.pi/2.0 * px_dist_center / 400.0
					theta_deg = theta_rad * 180.0/math.pi
					new_r_scaled = new_r * 0.928247332 # correction factor to make new_r_max = r_max_SIGMA_measured to avoid inflating image dimensions
					# Note on the above (^), the SIGMA lens projects wider on the sensor than an actual equisolid lens
					
					r = (-3.0 * 10.0**-6) * theta_deg**3.0 + 0.0002 * theta_deg**2.0 + 0.1354 * theta_deg + 0.0457 # based on measurements of SIGMA lens
					
					# new_row,new_col forms a vector. We just need to scale the vector based on r/new_r_scaled
					scale = r / new_r_scaled
					
					vector = (new_row+0.5-400.0, new_col+0.5-400.0) # vector relative to 800,800 to the pixel center
					
					# new vector length
					length = px_dist_center * scale
					
					# scaled vector of new length
					scaled_vector = (length * vector[0]/px_dist_center, length * vector[1]/px_dist_center)
					
					# put back into pixel array coordinate space
					row = scaled_vector[0] - 0.5 + 400.0
					col = scaled_vector[1] - 0.5 + 400.0
					
					# get interpolated pixel value
					try:
						pixel = fbilinear(tmp, row, col)
					except:
						print('Error: new_row %i; new_col %i.' % (new_row,new_col))
						sys.exit()
					
					# place in transformed image array
					tform_ar[new_row, new_col] = pixel
		transformed = Image.fromarray(tform_ar.astype('uint8'))

		# save as jpeg
		transformed.save(output_filename, quality=100, exif=im.info.get('exif'))
